@@ -13,7 +13,6 @@ import ignite
 import torch
 
 from ..utils import DelayedKeyboardInterrupt, config, logged
-from .nn import autodevice
 
 EPOCH_STARTED = ignite.engine.Events.EPOCH_STARTED
 EPOCH_COMPLETED = ignite.engine.Events.EPOCH_COMPLETED
@@ -372,26 +371,10 @@ class Model:
         self.net.device = device_backup
         self._trainer = trainer_backup
 
-    @staticmethod
-    def load(fname: os.PathLike) -> "Model":
+    def upgrade(self) -> None:
         r"""
-        Load model from file
-
-        Parameters
-        ----------
-        fname
-            Specifies path to the file
-
-        Returns
-        -------
-        model
-            Loaded model
+        Upgrade the model if generated by older versions
         """
-        fname = pathlib.Path(fname)
-        with fname.open("rb") as f:
-            model = dill.load(f)
-        model.net.device = autodevice()
-        return model
 
 
 @logged
